/*!
 * automation-extra-interception-proxy v0.8.0 by Utyfua
 * https://github.com/utyfua/automation-extra-interception-proxy
 * @license MIT
 */
'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var ProxyAgent = require('proxy-agent');
var got = require('got');
var parseDataURL = require('data-urls');
var EventEmitter = require('events');
var puppeteerToughCookieStore = require('puppeteer-tough-cookie-store');
var toughCookie = require('tough-cookie');
var puppeteerExtraPlugin = require('puppeteer-extra-plugin');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var ProxyAgent__default = /*#__PURE__*/_interopDefaultLegacy(ProxyAgent);
var got__default = /*#__PURE__*/_interopDefaultLegacy(got);
var parseDataURL__default = /*#__PURE__*/_interopDefaultLegacy(parseDataURL);
var EventEmitter__default = /*#__PURE__*/_interopDefaultLegacy(EventEmitter);

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

function __decorate(decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
}

const EXTRA_PLUGIN_ID = "InterceptionProxyPlugin";
const NOOP = () => { };
const INTERCEPTION_KEY_HOOK = 'interceptions';
const NPM_PACKAGE_NAME = 'automation-extra-interception-proxy';
const AUTHOR_REPOSITORY_URL = `https://github.com/utyfua/${NPM_PACKAGE_NAME}`;
const AUTHOR_NEW_ISSUE_URL = `${AUTHOR_REPOSITORY_URL}/issues/new`;

/**
 * Plugin mode for handling requests
 */
var RequestMode;
(function (RequestMode) {
    /**
     * Plugin will do nothing about original request
     */
    RequestMode["ignore"] = "ignore";
    /**
     * Plugin will just listen to the original request/response data and all requests will fulfilled by puppeteer itself.
     * But some plugin functionality can be unavailable.
     */
    RequestMode["native"] = "native";
    /**
     * Plugin will do all requests by himself. All plugin features will be available.
     */
    RequestMode["managed"] = "managed";
})(RequestMode || (RequestMode = {}));
/**
 * Current stage of the request
 */
var RequestStage;
(function (RequestStage) {
    // * We just got a new request from the puppeteer and getting additional information about it.
    // initial = 'initial',
    /**
     * We got a new request from the puppeteer witch includes all necessary information about.
     *
     * At this stage we can adjust request.
     */
    RequestStage["gotRequest"] = "gotRequest";
    /**
     * The request in requesting process
     *
     * At this stage we unable to adjust request but still have not response to go forward.
     */
    RequestStage["sentRequest"] = "sentRequest";
    /**
     * We got response from the request witch probably was modified by the user and now user can adjust the response.
     *
     * At this stage we can adjust response.
     * At this stage the user will unable to override the request anymore.
     */
    RequestStage["gotResponse"] = "gotResponse";
    /**
     * We sent final response of the request to the browser.
     *
     * Its too late to adjust request or response.
     */
    RequestStage["sentResponse"] = "sentResponse";
    /**
     * Page were closed and we unable do anything
     *
     * From technical perspective `sentResponse` looks just the same
     */
    RequestStage["closed"] = "closed";
})(RequestStage || (RequestStage = {}));

const baseHttpHandlerKey = 'baseHttpHandlerKey';
async function baseHttpHandler(request) {
    try {
        const { method, url, headers, body, timeout, agent, gotHooks } = request;
        if (!url.startsWith('http:') && !url.startsWith('https:'))
            return;
        const gotOptions = {
            // main request options
            method, url, headers, body, hooks: gotHooks,
            // secondary request options
            timeout,
            // another `got` options
            responseType: 'buffer',
            throwHttpErrors: false,
            ignoreInvalidCookies: true,
            followRedirect: false,
        };
        if (agent) {
            gotOptions.agent = {
                http: agent,
                https: agent,
            };
        }
        const gotResponse = await got__default["default"](gotOptions);
        return {
            status: gotResponse.statusCode,
            headers: gotResponse.headers,
            // contentType: gotResponse.headers['content-type'],
            body: gotResponse.body,
        };
    }
    catch (error) {
        request.recordWarning('Unable to get response for', [request.url, error]);
        // TODO: throw correct `abortReason`
        return {
            abortReason: 'connectionreset'
        };
    }
}

const dataUrlHandlerKey = 'dataUrl';
async function dataUrlHandler(request) {
    const { url } = request;
    if (!url.startsWith('data:'))
        return;
    const dataURL = parseDataURL__default["default"](url);
    if (!dataURL)
        return; // the result cannot be parsed as a data: URL.
    const contentType = dataURL.mimeType.toString();
    return {
        status: 200,
        headers: {
            'content-type': contentType
        },
        // contentType,
        body: dataURL.body,
    };
}

const defaultLogger = function defaultLogger({ level, message, error, meta }) {
    message = '[interceptions] ' + message.split('\n').join('. ');
    switch (level) {
        case 'error':
            if (error)
                console.error(error);
            console.log(message, ...meta);
            break;
        case 'warning':
            console.warn('Warning: ' + message, ...meta);
            break;
        case 'info':
            console.log(message, ...meta);
            break;
    }
};

const DefaultConfig = {
    cooperativePriority: undefined,
    requestMode: RequestMode.managed,
    logger: defaultLogger,
    proxy: null,
    agent: null,
    timeout: 30000,
    // attempts: 1,
    nativeContinueIfPossible: false,
    ignoreResponseBodyIfPossible: true,
    enableLegacyCookieHandling: false,
    gotHooks: {},
    requestHandlers: [
        {
            key: baseHttpHandlerKey,
            priority: -10,
            handler: baseHttpHandler,
        },
        {
            key: dataUrlHandlerKey,
            priority: -11,
            handler: dataUrlHandler,
        },
    ],
    requestListeners: [],
};
function applyConfigurableMixin(base) {
    class ConfigurableMixinBase extends base {
    }
    for (let key of Object.keys(DefaultConfig)) {
        Object.defineProperty(ConfigurableMixinBase.prototype, key, {
            get: function () {
                return this.__configGetter(key);
            },
            set: function (value) {
                return this.__configSetter(key, value);
            },
            // enumerable: true,
            // configurable: true
        });
    }
    function getBaseEventListenerConfig(eventName) {
        return {
            key: Symbol(`Unknown ${eventName}`),
            priority: 1,
            handler: NOOP,
        };
    }
    function getEventListenerConfig(eventName, args) {
        let options = getBaseEventListenerConfig(eventName);
        let shift = 0;
        for (let arg of args) {
            const type = typeof arg;
            if (shift <= 0 && ['symbol', 'string'].includes(type)) {
                options.key = arg;
                shift = 1;
                continue;
            }
            if (shift <= 1 && ['number'].includes(type)) {
                options.priority = arg;
                shift = 2;
                continue;
            }
            if (shift <= 2 && ['function'].includes(type)) {
                options.handler = arg;
                shift = 3;
                continue;
            }
            throw new Error(`Unknown argument[${type}]: ${arg}`);
        }
        return options;
    }
    /**
     * Plugin general configuration.
     */
    class ConfigurableMixin extends ConfigurableMixinBase {
        constructor() {
            super(...arguments);
            this.__local = {};
        }
        getLocalConfiguration() {
            return this.__local;
        }
        getParentConfiguration() {
            return this.__parent || {};
        }
        __configGetter(key) {
            // merge listener list by parent of default configuration
            if (['requestHandlers', 'requestListeners'].includes(key)) {
                const listeners = [...(this.__parent && this.__parent[key] || DefaultConfig[key])];
                const localListeners = this.__local[key];
                if (localListeners) {
                    localListeners.forEach(eventOptions => {
                        const existsIndex = listeners.findIndex(({ key }) => eventOptions.key == key);
                        if (existsIndex === -1) {
                            listeners.push(eventOptions);
                        }
                        else {
                            listeners[existsIndex] = eventOptions;
                        }
                    });
                }
                // sort by priority
                listeners.sort((a, b) => b.priority - a.priority);
                return listeners;
            }
            if (this.__local && key in this.__local)
                return this.__local[key];
            if (this.__parent && key in this.__parent)
                return this.__parent[key];
            return DefaultConfig[key];
        }
        __configSetter(key, value) {
            if (key === 'proxy') {
                const agent = value ? new ProxyAgent__default["default"](value) : null;
                this.__local.agent = agent;
            }
            if (key === 'agent') {
                this.__local.proxy = null;
            }
            this.__local[key] = value;
            // this.emit(key, value)
        }
        __getEventBase(eventName) {
            let eventStorage = this.__local[eventName];
            if (!eventStorage) {
                eventStorage = this.__local[eventName] = [];
            }
            return { eventStorage };
        }
        __addEvent(eventName, args) {
            const eventOptions = getEventListenerConfig(eventName, args);
            const { eventStorage } = this.__getEventBase(eventName);
            const existsIndex = eventStorage.findIndex(({ key }) => eventOptions.key == key);
            if (existsIndex === -1) {
                eventStorage.push(eventOptions);
            }
            else {
                eventStorage[existsIndex] = eventOptions;
            }
        }
        __deleteLocalEvent(eventName, target) {
            const { eventStorage } = this.__getEventBase(eventName);
            const findIndex = eventStorage.findIndex(({ key, handler }) => key === target || handler === target);
            if (!findIndex)
                return false;
            eventStorage.splice(findIndex, 1);
            return true;
        }
        addRequestHandler(...args) {
            this.__addEvent('requestHandlers', args);
        }
        deleteLocalRequestHandler(target) {
            return this.__deleteLocalEvent('requestHandlers', target);
        }
        addRequestListener(...args) {
            this.__addEvent('requestListeners', args);
        }
        deleteLocalRequestListener(target) {
            return this.__deleteLocalEvent('requestListeners', target);
        }
        waitForRequest(filter, options = {}) {
            return new Promise((_resolve, _reject) => {
                var _a;
                (_a = options.timeout) !== null && _a !== void 0 ? _a : (options.timeout = this.timeout);
                let resolved = false;
                const resolve = (error, result) => {
                    if (resolved)
                        return;
                    resolved = true;
                    this.deleteLocalRequestListener(listener);
                    if (timeout !== null) {
                        clearTimeout(timeout);
                    }
                    if (result === null || result === void 0 ? void 0 : result.request)
                        result.request.ignoreResponseBodyIfPossible = options.ignoreResponseBodyIfPossible || false;
                    if (!result) {
                        _reject(error || new Error('Unknown action'));
                    }
                    else {
                        _resolve(result);
                    }
                };
                const timeout = options.timeout ?
                    setTimeout(() => resolve(new Error(`Timeout ${options.timeout} ms`)), options.timeout) : null;
                async function listener(request) {
                    try {
                        if (await filter(request)) {
                            resolve(null, { request });
                        }
                    }
                    catch (error) {
                        resolve(error);
                    }
                }
                this.addRequestListener(options.priority || 0, listener);
            });
        }
        flushLocal(key) {
            if (key) {
                delete this.__local[key];
            }
            else {
                this.__local = {};
            }
        }
    }
    return ConfigurableMixin;
}

/**
 * @utyfua:
 * - I understand `.debug` exists for causes like this
 * but in my practice I should collect such information
 * and save in logs witch I can easily filter it after.
 */
function applyLoggableMixin(base) {
    class LoggableMixin extends base {
        recordError(message, error, ...meta) {
            this.logger({
                level: 'error',
                message,
                error,
                meta,
            });
        }
        recordInternalError(message, ...meta) {
            const error = new Error(message);
            this.recordError(message, error, ...meta);
        }
        recordWarning(message, ...meta) {
            this.logger({
                level: 'warning',
                message,
                meta,
            });
        }
    }
    return LoggableMixin;
}

function applyNetworkMixin(base) {
    /**
     * Expecting this.stage in runtime
     */
    class NetworkMixin extends base {
        get isRequestOverrideAvailable() {
            return [
                // RequestStage.initial,
                RequestStage.gotRequest,
            ].includes(this.stage);
        }
        get isResponseCollecting() {
            return this.stage === RequestStage.sentRequest;
        }
        get isResponseCollected() {
            return [
                RequestStage.gotResponse,
                RequestStage.sentResponse,
            ].includes(this.stage);
        }
        get isResponseOverrideAvailable() {
            return [
                RequestStage.gotRequest,
                RequestStage.sentRequest,
                RequestStage.gotResponse,
            ].includes(this.stage);
        }
        get isResponseFinished() {
            return [
                RequestStage.sentResponse,
                RequestStage.closed,
            ].includes(this.stage);
        }
        // todo: rework relations
        get json() {
            try {
                return JSON.parse(this.body);
            }
            catch (e) {
                return undefined;
            }
        }
        set json(obj) {
            this.body = JSON.stringify(obj);
        }
    }
    return NetworkMixin;
}

function getStageEnhancedErrorMessage(key, stage) {
    let message = 'unknown stage??';
    switch (stage) {
        // case RequestStage.initial:
        //     message = 'the request was not initialized properly yet';
        //     break;
        case RequestStage.gotRequest:
            message = 'the request was not initialized properly yet';
            break;
        case RequestStage.sentRequest:
            message = 'the request started to getting the response';
            break;
        case RequestStage.gotResponse:
            message = 'the request already got the response';
            break;
        case RequestStage.sentResponse:
            message = 'response for this request already were send to puppeteer';
            break;
        case RequestStage.closed:
            message = 'destination browser of page were already closed';
            break;
        default:
            message = 'the request currently in unknown stage';
            break;
    }
    return `Unable to enhance ${key} because ${message}`;
}

function getCDPSession(page, request) {
    // WARNING: we are using private property of Puppeteer.HTTPRequest here
    // @ts-ignore: _client is private but we want to get this variable
    let client = request._client;
    // for puppeteer@13 and below
    if (client)
        return client;
    // in another case we deal with private property #client
    // get frame' client if possible
    const frame = request.frame();
    if (frame) {
        // @ts-ignore: its inaccessible in types for puppeteer@13 and below
        return frame._client();
    }
    // in another case just return page' client
    // @ts-ignore: its inaccessible in types for puppeteer@13 and below
    return page._client();
}

let ResponseBase = class ResponseBase extends EventEmitter__default["default"] {
};
ResponseBase = __decorate([
    applyNetworkMixin,
    applyLoggableMixin,
    applyConfigurableMixin
], ResponseBase);
for (let key of ['status', 'headers', /*'contentType',*/ 'body', 'abortReason']) {
    Object.defineProperty(ResponseBase.prototype, key, {
        get: function () {
            if (key === 'body' && this.responseOptions._bodyError)
                throw new Error(this.responseOptions._bodyError);
            return this.responseOptions[key];
        },
        set: function (value) {
            return this.__responseOptionSetter(key, value);
        },
        enumerable: true,
        configurable: true
    });
}
class InterceptionProxyResponse extends ResponseBase {
    constructor(__parent, responseOptions) {
        super();
        this.originalResponse = undefined;
        this.__parent = __parent;
        this.responseOptions = responseOptions;
        this.emit2('responseOptions', this.responseOptions);
    }
    emit2(eventName, payload) {
        this.emit(eventName, payload);
        this.__parent.emit(eventName, payload);
    }
    static async __getResponse(request) {
        for (let handlerObj of request.requestHandlers) {
            try {
                const responseOptions = await handlerObj.handler(request);
                if (responseOptions)
                    return responseOptions;
            }
            catch (error) {
                request.recordError(`Interception handler "${String(handlerObj.key)}" throws an error. ` +
                    `For now we will ignore an error`, error);
            }
        }
        throw new Error(`No requestHandlers left. ` +
            `Looks like ${NPM_PACKAGE_NAME} doest not support urls like that` +
            `Please provide more information ${AUTHOR_NEW_ISSUE_URL}`);
    }
    static async proceedNewResponse(request, responseOptions) {
        try {
            responseOptions !== null && responseOptions !== void 0 ? responseOptions : (responseOptions = await this.__getResponse(request));
        }
        catch (error) {
            responseOptions = { abortReason: 'connectionfailed' };
            request.recordError(`Unable to proceed url`, error, request.url);
        }
        const response = new InterceptionProxyResponse(request, responseOptions);
        return response;
    }
    static async proceedOriginalResponse(request, originalResponse) {
        const responseOptions = {
            status: originalResponse.status(),
            headers: originalResponse.headers(),
            body: '',
        };
        try {
            if (!request.getLocalConfiguration().ignoreResponseBodyIfPossible) {
                responseOptions.body = await originalResponse.buffer();
            }
            else {
                responseOptions._bodyError = `${NPM_PACKAGE_NAME}: Response body was not resolved for performance. ` +
                    `Use 'ignoreResponseBodyIfPossible' option or 'await response.originalResponse.buffer()' to avoid this error`;
            }
        }
        catch (e) {
            responseOptions._bodyError = `${NPM_PACKAGE_NAME}: Unable to get original response`;
            // console.dir(e);
            // Could not load body for this request. This might happen if the request is a pre a preflight request.
        }
        const response = new InterceptionProxyResponse(request, responseOptions);
        response.originalResponse = originalResponse;
        return response;
    }
    getRequest() { return this.__parent; }
    get page() { return this.__parent.page; }
    get originalRequest() { return this.__parent.originalRequest; }
    get stage() { return this.__parent.stage; }
    set stage(value) { this.__parent.stage = value; }
    __responseOptionSetter(key, value) {
        if (!this.isResponseOverrideAvailable) {
            throw new Error(getStageEnhancedErrorMessage(key, this.stage));
        }
        if (key === 'abortReason') {
            this.responseOptions = { abortReason: value };
        }
        else {
            if ("abortReason" in this.responseOptions)
                delete this.responseOptions.abortReason;
            // const responseOptions = this.responseOptions as IResponseOverrides
            // @ts-ignore
            this.responseOptions[key] = value;
            // if (!responseOptions.status) responseOptions.status = 200;
            // if (!responseOptions.headers) responseOptions.headers = {};
            // if (!responseOptions.body) responseOptions.body = '';
        }
        this.emit2(key, value);
        this.emit2('responseOptions', this.responseOptions);
    }
    async setResponse(responseOptions) {
        this.responseOptions = responseOptions;
        return this;
    }
    async getResponse() {
        return this;
    }
    async abort(abortReason) {
        this.responseOptions = { abortReason };
        return this;
    }
    async continue() {
        // prevent processing for already handled requests
        if (!this.isResponseOverrideAvailable) {
            if (this.requestMode !== RequestMode.managed)
                return;
            // TODO: warn?
            return;
        }
        this.stage = RequestStage.sentResponse;
        const response = this.responseOptions;
        try {
            if ("body" in response) {
                const cookieHeaderRaw = response.headers["set-cookie"];
                const cookieHeader = typeof cookieHeaderRaw === 'string' ? [cookieHeaderRaw] : cookieHeaderRaw;
                if (this.enableLegacyCookieHandling && cookieHeader) {
                    response.headers["set-cookie"] = undefined;
                    const cookieJar = new toughCookie.CookieJar(new puppeteerToughCookieStore.PuppeteerToughCookieStore(getCDPSession(this.page, this.originalRequest)));
                    await Promise.all(cookieHeader.map(cookieString => {
                        const cookie = toughCookie.Cookie.parse(cookieString);
                        if (!cookie)
                            return;
                        return cookieJar.setCookie(cookie, this.getRequest().url);
                    }));
                }
                await this.originalRequest.respond(response, this.cooperativePriority);
            }
            else if ("continue" in response) {
                await this.originalRequest.continue(undefined, this.cooperativePriority);
            }
            else {
                await this.originalRequest.abort(response.abortReason, this.cooperativePriority);
            }
        }
        catch (error) {
            this.stage = RequestStage.closed;
            const client = getCDPSession(this.page, this.originalRequest);
            if (client && !client.connection()) ;
            else {
                this.emit2('error', error);
            }
        }
    }
}

function getDomain(urlString) {
    return new URL(urlString).hostname;
}
function getOrigin(urlString) {
    return new URL(urlString).origin;
}

function getFrameUrl(frame) {
    if (!frame)
        return null;
    const url = frame.url();
    if (url)
        return url;
    const parentFrame = frame.parentFrame();
    return getFrameUrl(parentFrame);
}
function adjustRequestCorsHeaders({ originalRequest }, requestOptions) {
    const frame = originalRequest.frame();
    const frameUrl = getFrameUrl(frame);
    if (!frameUrl || originalRequest.resourceType() !== "xhr")
        return;
    try {
        if (getDomain(frameUrl) === getDomain(requestOptions.url))
            return;
        requestOptions.headers['Origin'] = getOrigin(frameUrl);
    }
    catch (error) {
        console.error(error);
    }
}

/**
 * keep in track frow state
 *
 * @internal
 */
class _OriginalRequestStateManager {
    constructor(page, request) {
        this.page = page;
        this.request = request;
        this.userListeners = [];
        this.pageListeners = {
            response: (response) => {
                if (response.request() !== request)
                    return;
                this.close(response);
            },
            requestfailed: (_request) => {
                if (_request !== request)
                    return;
                this.close();
            },
            requestfinished: (_request) => {
                if (_request !== request)
                    return;
                this.close();
            },
        };
        this.setListeners('on');
    }
    setListeners(method) {
        Object.entries(this.pageListeners).forEach(([name, listener]) => 
        // @ts-ignore: should we do typings?
        this.page[method](name, listener));
    }
    onResponse(listener) {
        if (this.response !== undefined) {
            listener(this.response);
            return;
        }
        this.userListeners.push(listener);
    }
    close(response) {
        if (this.response !== undefined)
            return;
        const resultResponse = this.response = response || this.request.response();
        this.setListeners('off');
        const userListeners = this.userListeners;
        this.userListeners = [];
        userListeners.forEach(listener => listener(resultResponse));
    }
}

let RequestBase = class RequestBase extends EventEmitter__default["default"] {
};
RequestBase = __decorate([
    applyConfigurableMixin,
    applyLoggableMixin,
    applyNetworkMixin
], RequestBase);
for (let key of ['method', 'url', 'headers', 'body']) {
    Object.defineProperty(RequestBase.prototype, key, {
        get: function () {
            return this.requestOptions[key];
        },
        set: function (value) {
            return this.__requestOptionSetter(key, value);
        },
        enumerable: true,
        configurable: true
    });
}
/**
 * Plugin' request. The request have significant difference with Puppeteer' request.
 */
class InterceptionProxyRequest extends RequestBase {
    constructor(initial, requestOptions) {
        super();
        this.__isAdjusted = false;
        this.page = initial.page;
        this.originalRequest = initial.originalRequest;
        this.requestOptions = requestOptions;
        this.__stage = RequestStage.gotRequest;
        this.__parent = initial.__parent;
    }
    emit2(eventName, payload) {
        var _a;
        this.emit(eventName, payload);
        (_a = this.__response) === null || _a === void 0 ? void 0 : _a.emit(eventName, payload);
    }
    static async proceedNewRequest(initial) {
        const { page, originalRequest, __parent } = initial;
        const originalRequestStateManager = new _OriginalRequestStateManager(page, originalRequest);
        const requestOptions = {
            method: originalRequest.method(),
            url: originalRequest.url(),
            headers: Object.assign({}, originalRequest.headers()),
            body: originalRequest.postData(),
        };
        adjustRequestCorsHeaders(initial, requestOptions);
        const isRequestClientClosed = () => {
            const client = getCDPSession(page, originalRequest);
            return !client.connection();
        };
        try {
            if (__parent.enableLegacyCookieHandling) {
                const cookieJar = new toughCookie.CookieJar(new puppeteerToughCookieStore.PuppeteerToughCookieStore(getCDPSession(__parent.page, originalRequest)));
                requestOptions.headers['Cookie'] = await cookieJar.getCookieString(requestOptions.url);
            }
        }
        catch (error) {
            if (isRequestClientClosed())
                return;
            __parent.logger({
                level: 'warning',
                error: error,
                message: `Getting cookie error but we will proceed without\n` +
                    (error instanceof Error ? error.message : '' + error),
                meta: [requestOptions.url],
            });
        }
        const request = new InterceptionProxyRequest(initial, requestOptions);
        // native response listener handlers
        request.once('responseInstance', () => {
            originalRequestStateManager.close();
        });
        originalRequestStateManager.onResponse(async (originalResponse) => {
            if (request.__response)
                return;
            request.stage = RequestStage.sentRequest;
            const response = originalResponse ?
                await InterceptionProxyResponse.proceedOriginalResponse(request, originalResponse) :
                new InterceptionProxyResponse(request, { abortReason: 'failed' });
            request._setResponseInstance(response, RequestMode.native);
        });
        for (let handlerObj of request.requestListeners) {
            try {
                const localRes = await handlerObj.handler(request);
                if (localRes)
                    break;
            }
            catch (error) {
                if (isRequestClientClosed())
                    return;
                request.recordError(`Your interception handler "${String(handlerObj.key)}" throws an error. ` +
                    `Please wrap your function using try-catch blocks. For now we will ignore an error`, error);
            }
            if (isRequestClientClosed())
                return;
        }
        try {
            if (request.isResponseOverrideAvailable &&
                !('ignoreResponseBodyIfPossible' in request.getLocalConfiguration())) {
                request.ignoreResponseBodyIfPossible = request.ignoreResponseBodyIfPossible;
            }
            await request.continue();
        }
        catch (error) {
            if (isRequestClientClosed())
                return;
            request.recordError('Unable to proceed response from interception handler. We will abort the request.', error);
            try {
                await originalRequest.abort('failed', request.cooperativePriority);
            }
            catch (error) {
                // we already said that something went wrong
            }
        }
    }
    get stage() { return this.__stage; }
    ;
    set stage(value) {
        this.__stage = value;
        this.emit2('stage', value);
    }
    ;
    __requestOptionSetter(key, value) {
        if (!this.isRequestOverrideAvailable) {
            throw new Error(getStageEnhancedErrorMessage(key, this.stage));
        }
        if (this.requestMode !== RequestMode.managed) {
            this.recordWarning(`Possibly the request will not be upgraded by your "${key}" ` +
                `because current mode ${this.requestMode} does not support this ability. ` +
                `Please set "requestMode" variable to "${RequestMode.managed}" and then try again.`);
        }
        if (!this.__isAdjusted)
            this.__isAdjusted = true;
        this.requestOptions[key] = value;
        this.emit2(key, value);
        this.emit2('requestOptions', this.requestOptions);
    }
    async _getResponseInstance(responseOptions) {
        const { requestMode } = this;
        // @ts-ignore: we are not much care about ignoring requests
        if (requestMode === RequestMode.ignore)
            return null;
        if (this.isRequestOverrideAvailable && !responseOptions &&
            (this.requestMode === RequestMode.native ||
                (this.requestMode === RequestMode.managed && !this.__isAdjusted && this.nativeContinueIfPossible))) {
            this.stage = RequestStage.sentRequest;
            this.originalRequest.continue(undefined, this.cooperativePriority);
        }
        if (this.__response || this.isResponseCollecting ||
            (requestMode === RequestMode.native && !responseOptions)) {
            const response = this.__response || await new Promise(c => this.once('responseInstance', c));
            if (responseOptions) {
                response.setResponse(responseOptions);
            }
            return response;
        }
        this.stage = RequestStage.sentRequest;
        const response = await InterceptionProxyResponse.proceedNewResponse(this, responseOptions);
        this._setResponseInstance(response);
        return response;
    }
    ;
    // If its managed response we can modify data until it will be not sent to puppeteer,
    // but in another case it will be finale version of response
    _setResponseInstance(response, mode = RequestMode.managed) {
        this.__response = response;
        this.stage = mode === RequestMode.managed ? RequestStage.gotResponse : RequestStage.sentResponse;
        this.emit2('responseInstance', response);
    }
    getRequest() {
        return this;
    }
    ;
    async setResponse(response) {
        return await this._getResponseInstance(response);
    }
    async getResponse({ requestMode } = {}) {
        if (requestMode)
            this.requestMode = requestMode;
        return await this._getResponseInstance();
    }
    async abort(abortReason) {
        return await this._getResponseInstance({ abortReason: abortReason || 'aborted' });
    }
    async continue() {
        const response = await this.getResponse();
        await response.continue();
    }
}

let PageBase = class PageBase {
};
PageBase = __decorate([
    applyConfigurableMixin,
    applyLoggableMixin
], PageBase);
class InterceptionProxyPageConfig extends PageBase {
    constructor(page, __parent) {
        super();
        this.page = page;
        this.__parent = __parent;
    }
    static async proceedNewPage(page, __parent) {
        if (page[INTERCEPTION_KEY_HOOK]) {
            throw new Error('Interception hook already registered.');
        }
        const interceptionConfig = new InterceptionProxyPageConfig(page, __parent);
        page[INTERCEPTION_KEY_HOOK] = interceptionConfig;
        page.on('request', (originalRequest) => {
            InterceptionProxyRequest.proceedNewRequest({
                page: interceptionConfig.page,
                __parent: interceptionConfig,
                originalRequest
            });
        });
        // sad, but puppeteer-extra will not wait our promise. So we are doing that in the end
        await page.setRequestInterception(true);
        return interceptionConfig;
    }
}

let BrowserBase = class BrowserBase {
};
BrowserBase = __decorate([
    applyConfigurableMixin,
    applyLoggableMixin
], BrowserBase);
class InterceptionProxyBrowserConfig extends BrowserBase {
    constructor(__parent) {
        super();
        this.__parent = __parent;
    }
    async proceedNewPage(page) {
        return await InterceptionProxyPageConfig.proceedNewPage(page, this);
    }
}

let PluginBase = class PluginBase extends puppeteerExtraPlugin.PuppeteerExtraPlugin {
    get name() {
        return EXTRA_PLUGIN_ID;
    }
};
PluginBase = __decorate([
    applyConfigurableMixin,
    applyLoggableMixin
], PluginBase);
class InterceptionProxyPlugin extends PluginBase {
    async onPageCreated(page) {
        const browser = await page.browser();
        browser[INTERCEPTION_KEY_HOOK].proceedNewPage(page);
    }
    // Add additions to already existing pages and frames
    async onBrowser(browser) {
        browser[INTERCEPTION_KEY_HOOK] = new InterceptionProxyBrowserConfig(this);
        const pages = await browser.pages();
        for (const page of pages) {
            this.onPageCreated(page);
        }
    }
}

/**
 * Add interception ability to the page [(sample)](https://github.com/utyfua/automation-extra-interception-proxy/blob/master/samples/singlePageInterception.js)
 *
 * @param page Page for future interceptions
 */
async function wrapPage(page, config) {
    return await InterceptionProxyPageConfig.proceedNewPage(page, config);
}

exports.AutomationExtraInterceptionProxyPlugin = InterceptionProxyPlugin;
exports.InterceptionProxyPlugin = InterceptionProxyPlugin;
exports["default"] = InterceptionProxyPlugin;
exports.wrapPage = wrapPage;


  module.exports = exports.default || {}
  Object.entries(exports).forEach(([key, value]) => { module.exports[key] = value })
//# sourceMappingURL=index.cjs.js.map
