"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.applyConfigurableMixin = void 0;
const proxy_agent_1 = __importDefault(require("proxy-agent"));
const interfaces_1 = require("../interfaces");
const baseHttpHandler_1 = require("../handlers/baseHttpHandler");
const dataUrl_1 = require("../handlers/dataUrl");
const logger_1 = require("../utils/logger");
const DefaultConfig = {
    cooperativePriority: undefined,
    requestMode: interfaces_1.RequestMode.managed,
    logger: logger_1.defaultLogger,
    proxy: null,
    agent: null,
    timeout: 30000,
    // attempts: 1,
    nativeContinueIfPossible: false,
    ignoreResponseBodyIfPossible: true,
    enableLegacyCookieHandling: false,
    gotHooks: {},
    requestHandlers: [
        {
            key: baseHttpHandler_1.baseHttpHandlerKey,
            priority: -10,
            handler: baseHttpHandler_1.baseHttpHandler,
        },
        {
            key: dataUrl_1.dataUrlHandlerKey,
            priority: -11,
            handler: dataUrl_1.dataUrlHandler,
        },
    ],
    requestListeners: [],
};
function applyConfigurableMixin(base) {
    class ConfigurableMixinBase extends base {
    }
    for (let key of Object.keys(DefaultConfig)) {
        Object.defineProperty(ConfigurableMixinBase.prototype, key, {
            get: function () {
                return this.__configGetter(key);
            },
            set: function (value) {
                return this.__configSetter(key, value);
            },
            // enumerable: true,
            // configurable: true
        });
    }
    function getBaseEventListenerConfig(eventName) {
        return {
            key: Symbol(`Unknown ${eventName}`),
            priority: 1,
            handler: interfaces_1.NOOP,
        };
    }
    function getEventListenerConfig(eventName, args) {
        let options = getBaseEventListenerConfig(eventName);
        let shift = 0;
        for (let arg of args) {
            const type = typeof arg;
            if (shift <= 0 && ['symbol', 'string'].includes(type)) {
                options.key = arg;
                shift = 1;
                continue;
            }
            if (shift <= 1 && ['number'].includes(type)) {
                options.priority = arg;
                shift = 2;
                continue;
            }
            if (shift <= 2 && ['function'].includes(type)) {
                options.handler = arg;
                shift = 3;
                continue;
            }
            throw new Error(`Unknown argument[${type}]: ${arg}`);
        }
        return options;
    }
    /**
     * Plugin general configuration.
     */
    class ConfigurableMixin extends ConfigurableMixinBase {
        constructor() {
            super(...arguments);
            this.__local = {};
        }
        getLocalConfiguration() {
            return this.__local;
        }
        getParentConfiguration() {
            return this.__parent || {};
        }
        __configGetter(key) {
            // merge listener list by parent of default configuration
            if (['requestHandlers', 'requestListeners'].includes(key)) {
                const listeners = [...(this.__parent && this.__parent[key] || DefaultConfig[key])];
                const localListeners = this.__local[key];
                if (localListeners) {
                    localListeners.forEach(eventOptions => {
                        const existsIndex = listeners.findIndex(({ key }) => eventOptions.key == key);
                        if (existsIndex === -1) {
                            listeners.push(eventOptions);
                        }
                        else {
                            listeners[existsIndex] = eventOptions;
                        }
                    });
                }
                // sort by priority
                listeners.sort((a, b) => b.priority - a.priority);
                return listeners;
            }
            if (this.__local && key in this.__local)
                return this.__local[key];
            if (this.__parent && key in this.__parent)
                return this.__parent[key];
            return DefaultConfig[key];
        }
        __configSetter(key, value) {
            if (key === 'proxy') {
                const agent = value ? new proxy_agent_1.default(value) : null;
                this.__local.agent = agent;
            }
            if (key === 'agent') {
                this.__local.proxy = null;
            }
            this.__local[key] = value;
            // this.emit(key, value)
        }
        __getEventBase(eventName) {
            let eventStorage = this.__local[eventName];
            if (!eventStorage) {
                eventStorage = this.__local[eventName] = [];
            }
            return { eventStorage };
        }
        __addEvent(eventName, args) {
            const eventOptions = getEventListenerConfig(eventName, args);
            const { eventStorage } = this.__getEventBase(eventName);
            const existsIndex = eventStorage.findIndex(({ key }) => eventOptions.key == key);
            if (existsIndex === -1) {
                eventStorage.push(eventOptions);
            }
            else {
                eventStorage[existsIndex] = eventOptions;
            }
        }
        __deleteLocalEvent(eventName, target) {
            const { eventStorage } = this.__getEventBase(eventName);
            const findIndex = eventStorage.findIndex(({ key, handler }) => key === target || handler === target);
            if (!findIndex)
                return false;
            eventStorage.splice(findIndex, 1);
            return true;
        }
        addRequestHandler(...args) {
            this.__addEvent('requestHandlers', args);
        }
        deleteLocalRequestHandler(target) {
            return this.__deleteLocalEvent('requestHandlers', target);
        }
        addRequestListener(...args) {
            this.__addEvent('requestListeners', args);
        }
        deleteLocalRequestListener(target) {
            return this.__deleteLocalEvent('requestListeners', target);
        }
        waitForRequest(filter, options = {}) {
            return new Promise((_resolve, _reject) => {
                var _a;
                (_a = options.timeout) !== null && _a !== void 0 ? _a : (options.timeout = this.timeout);
                let resolved = false;
                const resolve = (error, result) => {
                    if (resolved)
                        return;
                    resolved = true;
                    this.deleteLocalRequestListener(listener);
                    if (timeout !== null) {
                        clearTimeout(timeout);
                    }
                    if (result === null || result === void 0 ? void 0 : result.request)
                        result.request.ignoreResponseBodyIfPossible = options.ignoreResponseBodyIfPossible || false;
                    if (!result) {
                        _reject(error || new Error('Unknown action'));
                    }
                    else {
                        _resolve(result);
                    }
                };
                const timeout = options.timeout ?
                    setTimeout(() => resolve(new Error(`Timeout ${options.timeout} ms`)), options.timeout) : null;
                async function listener(request) {
                    try {
                        if (await filter(request)) {
                            resolve(null, { request });
                        }
                    }
                    catch (error) {
                        resolve(error);
                    }
                }
                this.addRequestListener(options.priority || 0, listener);
            });
        }
        flushLocal(key) {
            if (key) {
                delete this.__local[key];
            }
            else {
                this.__local = {};
            }
        }
    }
    return ConfigurableMixin;
}
exports.applyConfigurableMixin = applyConfigurableMixin;
//# sourceMappingURL=ConfigurableMixin.js.map